package com.caimei365.user.service.impl;

import com.caimei365.user.components.RedisService;
import com.caimei365.user.components.WeChatService;
import com.caimei365.user.mapper.HeHeMapper;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.model.dto.HeHeUserDto;
import com.caimei365.user.model.vo.HeHeUserVo;
import com.caimei365.user.service.HeHeService;
import com.caimei365.user.utils.AliyunSmsUtil;
import com.caimei365.user.utils.CodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/4/21
 */
@Slf4j
@Service
public class HeHeServiceImpl implements HeHeService {
    @Resource
    private HeHeMapper heHeMapper;
    @Resource
    private WeChatService weChatService;
    @Resource
    private RedisService redisService;

    @Override
    public ResponseJson<HeHeUserVo> authorization(String code, HttpHeaders headers) {
        // 小程序微信授权获取登录信息
        ResponseJson<Map<String, Object>> appletsInfo = weChatService.getInfoMapByApplets(code, headers, 2);
        if (appletsInfo.getCode() == -1) {
            return ResponseJson.error(appletsInfo.getMsg(), null);
        }
        Map<String, Object> infoData = appletsInfo.getData();
        String openId = (String) infoData.get(WeChatService.Keys.OPEN_ID);
        HeHeUserVo heHeUser = heHeMapper.findHeHeUserByOpenId(openId);
        if (heHeUser == null) {
            heHeUser = new HeHeUserVo();
            heHeUser.setOpenId(openId);
        }
        return ResponseJson.success(heHeUser);
    }

    @Override
    public ResponseJson<HeHeUserVo> mobileLogin(HeHeUserDto heHeUserDto) {
        String redisSmsCode = (String) redisService.get("heCode" + heHeUserDto.getMobile());
        if (null != redisSmsCode && !"null".equals(redisSmsCode)) {
            if (redisSmsCode.equals(heHeUserDto.getVerificationCode())) {
                HeHeUserVo heHeUser = heHeMapper.findHeHeUserByMobile(heHeUserDto.getMobile());
                if (heHeUser != null) {
                    if (StringUtils.isBlank(heHeUser.getOpenId())) {
                        heHeUser.setOpenId(heHeUserDto.getOpenId());
                        heHeUser.setHeadImgUrl(heHeUserDto.getHeadImgUrl());
                        heHeUser.setNickName(heHeUserDto.getNickName());
                        heHeMapper.updateHeHeUser(heHeUser);
                    }
                } else {
                    heHeMapper.insertHeHeUser(heHeUserDto);
                    heHeUser = heHeMapper.findHeHeUserByMobile(heHeUserDto.getMobile());
                }
                return ResponseJson.success(heHeUser);
            } else {
                return ResponseJson.error("验证码错误", null);
            }
        }
        return ResponseJson.error("验证码已过期", null);
    }

    @Override
    public ResponseJson<String> sendVerificationCode(String mobile) {
        String randomCode = CodeUtil.generateCodeInt(6);
        boolean smsFlag = AliyunSmsUtil.sendSms(mobile, 7, "{code:" + randomCode + "}");
        if (!smsFlag) {
            // 短信发送失败重试一次
            AliyunSmsUtil.sendSms(mobile, 7, "{code:" + randomCode + "}");
        }
        log.info("呵呵商城注册验证码:" + randomCode);
        redisService.set("heCode:" + mobile, randomCode, 300L);
        return ResponseJson.success("发送成功");
    }
}
